selfDir_3d1bc249 := $(dir $(lastword $(MAKEFILE_LIST)))

# Make
addBeforeExt = $(basename $(2))$(1)$(suffix $(2))

# Commands
DIFF = diff --unified=2 $(+:_%=)

table = $(shell s="$(*F)"; echo "$${s\#\#*.}")# remove sort order prefix

vegxMaps := $(wildcard map.VegX.*.csv)
vegbienMaps := $(subst .VegX.,.VegBIEN.,$(vegxMaps))
tables := $(vegbienMaps:map.VegBIEN.%.csv=%)

root := $(selfDir_3d1bc249)..
psqlAsBien := $(root)/bin/psql_vegbien

#####

all: _always maps import verify

.SUFFIXES:

_always:
.PHONY: _always

clean: _always
	$(RM) $(all)

rm-%: _always
	$(RM) $*

%.out: % _always
	./$* >$@
.PRECIOUS: %.out

#####

reinstall: _always uninstall install

#####

maps: $(vegbienMaps) _always

all += $(vegbienMaps)

map.VegBIEN.%.csv: map.VegX.%.csv
	$(root)/bin/join_sort <$< $(root)/mappings/VegX-VegBIEN.$(table).csv >$@
.PRECIOUS: map.VegBIEN.%.csv

#####

import: _always $(addprefix import-,$(tables))

log = $*$(if $(n),.n=$(n),).log

logs := $(wildcard *.log)

rm_logs: _always
	$(RM) $(logs)

#####

verify: _always $(addprefix verify-,$(tables))

verify-%: verify.%.ref verify.%.out _always
	$(DIFF)

%.out: %.sql _always
	$(psqlAsBien) --no-align --field-separator='	' --pset=footer=off <$< >$@
.PRECIOUS: %.out

all += $(wildcard *.out)

#####

test: _always $(addprefix test-,$(tables))

test-%: test.%.ref test.%.out _always
	$(DIFF)

all += $(filter-out %.ref,$(wildcard test.*.out))

#####

ifdef db

mapEnv := in_engine=$(dbEngine) in_database=$(db)

####

ifeq ($(dbEngine),MySQL)

bienPassword := $(shell cat $(root)/config/bien_password)
mysqlAs = mysql --user=$(1) --password='$(bienPassword)'
mysqlAsRoot := $(call mysqlAs,root)
mysqlAsBien := $(call mysqlAs,bien)

import-%: map.VegBIEN.%.csv _always
	-$(import_cmd)
import_cmd = (set -x; "time" env commit=1 $(map) $<) \
$(if $(n),,>>$(log)) 2>&1$(if $(n),|tee -a $(log))

%.ref: %.ref.sql
	$(mysqlAsBien) $(db) <$< >$@
.PRECIOUS: %.ref

###

install: _always db

uninstall: _always rm_db

db: src.$(db).sql _always
	-$(mysqlAsRoot) <$<
	echo "GRANT SELECT ON $(db).* TO 'bien'@'localhost';"|$(mysqlAsRoot)
# ignore errors in db import so that GRANT will still be run

rm_db: _always
	-echo "REVOKE ALL ON $(db).* FROM 'bien'@'localhost';"|$(mysqlAsRoot)
	echo "DROP DATABASE IF EXISTS $(db);"|$(mysqlAsRoot)
# ignore errors if grant not defined

###

else
$(error dbEngine variable must be one of the following: MySQL, PostgreSQL)
endif

####

test.%.out: map.VegBIEN.%.csv _always
	env n=2 $(map) $< >$@ 2>&1
.PRECIOUS: test.%.out

#####

else
$(error db variable must be set)
endif

#####

# Must come after mapEnv is set
map := env $(mapEnv) out_database=vegbien $(root)/map
